from .lib import settrade_open_api as soa
from .util import send_python_error

class MarketData:
	def __init__(self, context, logger):
		self._context = context
		self._logger = logger
  
	def get_candlestick(
  	self,
		symbol,
   	interval,
		limit=500,
		start="",
		end="",
		is_adjusted=True,
  ):
		current_time = self._logger.get_current_time()

		try:
			interval_list = ["1m", "3m", "5m", "10m", "15m", "30m", "60m", "1d", "1w", "1M"]
  
			if interval not in interval_list:
				raise ValueError(interval + " is invalid. Please choose " + str(interval_list))
    
			result = soa.get_candlesticks(
				self._context,
				interval,
				symbol,
				limit,
				is_adjusted,
				start,
				end,
			)
   
			self._logger.write(current_time, "PYTHON_INFO", "get_candlesticks", "success")
   
			return result
		except Exception as err:
			self._logger.write(current_time, "PYTHON_EXCEPTION", "get_candlesticks", err)
   
			return send_python_error(err)


from ...lib import settrade_open_api as soa
from ...util import send_python_error

class MarketRepDerivatives:
    def __init__(self, context, logger):
        self._context = context
        self._logger = logger

    def get_account_info(self, account_no):
        start = self._logger.get_current_time()
        try:
            result = soa.get_marketrep_derivatives_account_info(self._context, account_no)
            self._logger.write(start, "PYTHON_INFO", "get_account_info", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "get_account_info", err)
            return send_python_error(err)

    def get_orders(self):
        start = self._logger.get_current_time()
        try:
            result = soa.get_marketrep_derivatives_orders(self._context)
            self._logger.write(start, "PYTHON_INFO", "get_orders", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "get_orders", err)
            return send_python_error(err)

    def get_order(self, order_no):
        start = self._logger.get_current_time()
        try:
            result = soa.get_marketrep_derivatives_order(self._context, order_no)
            self._logger.write(start, "PYTHON_INFO", "get_order", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "get_order", err)
            return send_python_error(err)

    def get_orders_by_account_no(self, account_no):
        start = self._logger.get_current_time()
        try:
            result = soa.get_marketrep_derivatives_orders_by_account_no(
                self._context, account_no
            )
            self._logger.write(
                start, "PYTHON_INFO", "get_orders_by_account_no", "success"
            )
            return result
        except Exception as err:
            self._logger.write(
                start, "PYTHON_EXCEPTION", "get_orders_by_account_no", err
            )
            return send_python_error(err)

    def get_portfolio(self, account_no):
        start = self._logger.get_current_time()
        try:
            result = soa.get_marketrep_derivatives_portfolios(self._context, account_no)
            self._logger.write(start, "PYTHON_INFO", "get_portfolio", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "get_portfolio", err)
            return send_python_error(err)

    def get_trades(self, account_no):
        start = self._logger.get_current_time()
        try:
            result = soa.get_marketrep_derivatives_trades(self._context, account_no)
            self._logger.write(start, "PYTHON_INFO", "get_trades", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "get_trades", err)
            return send_python_error(err)

    def place_order(
        self,
        account_no,
        symbol,
        price,
        volume,
        side,
        position,
        price_type="LIMIT",
        validity_type="GOOD_TILL_DAY",
        stop_condition="",
        stop_symbol="",
        stop_price=0,
        bypass_warning=False,
    ):
        start = self._logger.get_current_time()
        try:
            result = soa.place_marketrep_derivatives_order(
                self._context,
                account_no,
                symbol,
                price,
                volume,
                side,
                position,
                price_type,
                validity_type,
                stop_condition,
                stop_symbol,
                stop_price,
                bypass_warning,
            )
            self._logger.write(start, "PYTHON_INFO", "place_order", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "place_order", err)
            return send_python_error(err)

    def cancel_order(self, account_no, order_no):
        start = self._logger.get_current_time()
        try:
            result = soa.cancel_marketrep_derivatives_order(
                self._context, account_no, order_no
            )
            self._logger.write(start, "PYTHON_INFO", "cancel_order", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "cancel_order", err)
            return send_python_error(err)

    def change_order(
        self, account_no, order_no, new_account_no="", new_price=0, new_volume=0,
    ):
        new_position = ""
        new_price_type = ""
        start = self._logger.get_current_time()
        try:
            result = soa.change_marketrep_derivatives_order(
                self._context,
                account_no,
                order_no,
                new_account_no,
                new_price,
                new_volume,
                new_position,
                new_price_type,
            )
            self._logger.write(start, "PYTHON_INFO", "change_order", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "change_order", err)
            return send_python_error(err)

    def cancel_orders(self, account_no, orders_no):
        start = self._logger.get_current_time()
        try:
            result = soa.cancel_marketrep_derivatives_orders(
                self._context, account_no, orders_no
            )
            self._logger.write(start, "PYTHON_INFO", "cancel_orders", "success")
            return result
        except Exception as err:
            self._logger.write(start, "PYTHON_EXCEPTION", "cancel_orders", err)
            return send_python_error(err)

    def place_trade_report(
        self, symbol, cpm, price, volume, position, tr_type, buyer="", seller=""
    ):
        start = self._logger.get_current_time()
        try:
            result = soa.place_marketrep_derivatives_trade_report(
                self._context,
                symbol,
                cpm,
                price,
                volume,
                position,
                tr_type,
                buyer,
                seller,
            )
            self._logger.write(
                start,
                "PYTHON_INFO",
                "place_trade_report",
                "success",
            )
            return result
        except Exception as err:
            self._logger.write(
                start,
                "PYTHON_EXCEPTION",
                "place_trade_report",
                err,
            )
            return send_python_error(err)



from .user import User
from ..account.marketrep.derivativesaccount import MarketRepDerivatives
from ..account.marketrep.equityaccount import MarketRepEquity
from ..logger import Logger

class MarketRep(User):
    def __init__(self, app_id, app_secret, app_code, broker_id, is_auto_queue=False):
        self._logger = Logger("settrade_log")
        self._logger.create_log_dir()
        
        start = self._logger.get_current_time()
        
        try:
            super().__init__(app_id, app_secret, app_code, broker_id, is_auto_queue)
            self._logger.remove_expired_log()
            self._logger.write(start, "PYTHON_INFO", "Login_MarketRep", self._context)
        except Exception as e:
            self._logger.write(start, "PYTHON_EXCEPTION", "Login_MarketRep", e)
            raise Exception(e)

    def Derivatives(self):
        return MarketRepDerivatives(self._context, self._logger)

    def Equity(self):
        return MarketRepEquity(self._context, self._logger)



from .user import User
from ..account.investor.derivativesaccount import InvestorDerivatives
from ..account.investor.equityaccount import InvestorEquity
from ..logger import Logger

class Investor(User):
    def __init__(self, app_id, app_secret, app_code, broker_id, is_auto_queue=False):
        self._logger = Logger("settrade_log")
        self._logger.create_log_dir()
        
        start = self._logger.get_current_time()
        
        try:
            super().__init__(app_id, app_secret, app_code, broker_id, is_auto_queue)
            self._logger.remove_expired_log()
            self._logger.write(start, "PYTHON_INFO", "Login_Investor", self._context)
        except Exception as e:
            self._logger.write(start, "PYTHON_EXCEPTION", "Login_Investor", e)
            raise Exception(e)

    def Derivatives(self, account_no):
        return InvestorDerivatives(self._context, self._logger, account_no)

    def Equity(self, account_no):
        return InvestorEquity(self._context, self._logger, account_no)



import os
import time
import datetime
import sys
from pathlib import Path

class Logger:
    def __init__(self, config_dir):
        self.__config_filename = "settradesdk_config.txt"
        self.__config_path = self.__userHomeDir()
        self.__config_dir = config_dir

    def get_current_time(self):
        return time.time()

    def write(self, start_time, level, topic, message):
        start_datetime_str = self.__epoch_to_datetime_str(start_time)
        file = open(self.__get_current_log_file(), "a")
        elap_time = self.__get_elap_time(start_time, self.get_current_time())
        level = "[" + level + "]"
        start_datetime_str = "[" + start_datetime_str + "]"
        seperate = " - "
        elap_time = "[Elapsed time " + elap_time + "s]"
        file.write(start_datetime_str)
        file.write(seperate)
        file.write(elap_time)
        file.write(topic)
        file.write(seperate)
        file.write(str(message))
        file.write(seperate)
        file.write(level)
        file.write("\n")
        file.close()

    def create_log_dir(self):
        try:
            os.mkdir(str(Path(self.__config_path) / Path(self.__config_dir)))
        except Exception as e:
            pass

    def remove_expired_log(self):
        self.__days = self.__get_config_max_log_days(
            str(Path(self.__config_path) / Path(self.__config_filename))
        )
        # get all log files
        dir_path = str(Path(self.__config_path) / Path(self.__config_dir))
        files = os.listdir(dir_path)
        current_time = self.get_current_time()
        minus_time = self.__get_minus_time(self.__days)
        # get expired files
        expired_files = [
            file
            for file in files
            if (
                self.__is_expired(
                    self.__get_last_modified(str(Path(dir_path) / Path(file))),
                    current_time,
                    minus_time,
                )
            )
        ]
        # remove all expired files
        for expired_file in expired_files:
            self.__remove_file(dir_path, expired_file)

    def __get_config_max_log_days(self, config_full_path):
        file = open(config_full_path, "r")
        lines = file.readlines()
        for line in lines:
            try:
                if "clear_log=" in line:
                    line = line.replace("clear_log=", "")
                    line = line.replace(" ", "")
                    return int(line)
            except:
                print("clear_log must be integer! assume to be 30 days")
                return 30

    def __epoch_to_datetime_str(self, epoch):
        return datetime.datetime.fromtimestamp(epoch).strftime("%Y-%m-%d %H:%M:%S")

    def __get_minus_time(self, days):
        return days * 86400

    def __get_last_modified(self, path):
        return os.path.getmtime(path)

    def __remove_file(self, dir_path, file):
        os.remove(str(Path(dir_path) / Path(file)))

    def __is_expired(self, last_modified_time, current_time, minus_time):
        return last_modified_time < current_time - minus_time

    def __get_current_datetime_str(self):
        return datetime.datetime.today().strftime("%Y-%m-%d")

    def __get_elap_time(self, start_time, end_time):
        elap = end_time - start_time
        return "%.6f" % elap

    def __get_current_log_file(self):
        log_file = self.__get_current_datetime_str() + ".txt"
        return str(Path(self.__config_path) / Path(self.__config_dir) / Path(log_file))

    def __userHomeDir(self):
        path = ""
        home = ""
        postfix = ""

        if sys.platform == "win32":
            postfix = "/AppData"

        home = str(os.getenv("HOMEDRIVE")) + str(os.getenv("HOMEPATH"))
        if home == "NoneNone":
            home = os.getenv("USERPROFILE")
        if home == None:
            home = os.getenv("HOME")

        if home == "":
            path = "./"
        else:
            path = home + postfix
        return path



from ..lib import settrade_open_api as soa
from ..mqtt import MQTTWebsocket
from ..market_data import MarketData

class User:
    def __init__(self, app_id, app_secret, app_code, broker_id, is_auto_queue=False):
        self._context = soa.login(app_id, app_secret, app_code, broker_id, is_auto_queue)

    def RealtimeDataConnection(self):
        return MQTTWebsocket(self._context)

    def MQTTWebsocket(self):
        return MQTTWebsocket(self._context)
    
    def MarketData(self):
        return MarketData(self._context, self._logger)



from .lib import settrade_open_api as soa
from threading import Thread, Event
import json

key_one_topic = "ONE_TOPIC"
key_two_topic = "TWO_TOPIC"

class Subscriber:
    def __init__(self, context, id, cb_func, topic1, topic2, on_message, parse_float_func, args):
        self._e = Event()

        def cpp_callback(result):
            try:
                result = json.loads(result)
                result = parse_float_func(result)
                on_message(result, self, *args)
            except Exception as e:
                print(e)
                self.stop()

        self._t = None

        def t_callback():
            key = cb_func(context, topic1, cpp_callback) if id == key_one_topic else cb_func(context, topic1, topic2, cpp_callback)
                       
            # wait event
            self._e.wait()
            # after kill event
            soa.unsubscribe(context["channel_id"], key)

        self._t = Thread(target=t_callback)

    def start(self):
        self._t.start()

    def stop(self):
        self._e.set()
        self._t.join()


class MQTTWebsocket:
    def __init__(self, context):
        self._context = context

    def subscribe_bid_offer(self, symbol, on_message, args=()):
        def parse_float(result):
            if result["is_success"]:
                for i in ["bid_price1", "bid_price2", "bid_price3", "bid_price4", "bid_price5", "ask_price1", "ask_price2", "ask_price3", "ask_price4", "ask_price5"]:
                    result["data"][i] = float(result["data"][i])
            return result

        return Subscriber(
            self._context, key_one_topic, soa.subscribe_bid_offer, symbol, None, on_message, parse_float, args
        )
        
    def subscribe_candlestick(self, symbol, interval, on_message, args=()):
        interval_list = ["1m", "3m", "5m", "10m", "15m", "30m", "60m", "1d", "1w", "1M"]

        if interval not in interval_list:
            raise ValueError(interval + " is invalid. Please choose " + str(interval_list))

        def parse_float(result):
            if result["is_success"]:
                for i in ["open", "high", "low", "close"]:
                    result["data"][i] = float(result["data"][i])
            return result

        return Subscriber(
            self._context, key_two_topic, soa.subscribe_candlestick, symbol, interval, on_message, parse_float, args
        )
        
    def subscribe_price_info(self, symbol, on_message, args=()):
        def parse_float(result):
            if result["is_success"]:
                for i in ["high", "low", "last", "projected_open_price"]:
                    result["data"][i] = float(result["data"][i])
            return result

        return Subscriber(
            self._context, key_one_topic, soa.subscribe_price_info, symbol, None, on_message, parse_float, args
        )

    def subscribe_derivatives_order(self, account_no, on_message, args=()):
        def parse_float(result):
            if result["is_success"]:
                for i in ["price"]:
                    result["data"][i] = float(result["data"][i])
            return result

        return Subscriber(
            self._context, key_one_topic, soa.subscribe_derivatives_order, account_no, None, on_message, parse_float, args
        )

    def subscribe_equity_order(self, account_no, on_message, args=()):
        def parse_float(result):
            if result["is_success"]:
                for i in ["price"]:
                    result["data"][i] = float(result["data"][i])
            return result

        return Subscriber(
            self._context, key_one_topic, soa.subscribe_equity_order, account_no, None, on_message, parse_float, args
        )



import sys
import platform
import distutils.sysconfig

from setuptools import setup, find_packages

platform_supported = False

dir_path = "src/settrade_open_api_uat/open_api"
lib_path = dir_path + "/lib"
wrapper_path = dir_path + "/wrapper"

py_ver = str(sys.version_info.major) + str(sys.version_info.minor)

if sys.platform == "win32":
    platform_supported = True

    if platform.architecture()[0] == "64bit":
        lib_path += "/windows_x64/python" + py_ver
    else:
        lib_path += "/windows_x86/python" + py_ver
elif sys.platform == "darwin":
    platform_supported = True
    lib_path += "/mac/python" + py_ver
elif sys.platform == "linux":
    platform_supported = True
    lib_path += "/linux/python" + py_ver

if not platform_supported:
    raise NotImplementedError(sys.platform)

setup(
    name="settrade",
    version="0.4.1",
    install_requires=["numpy"],
    packages=[
        "settrade.openapi",
        "settrade.openapi.account.investor",
        "settrade.openapi.account.marketrep",
        "settrade.openapi.user",
        "settrade.openapi.lib",
    ],
    package_dir={
        "settrade.openapi": wrapper_path,
        "settrade.openapi.account.investor": wrapper_path + "/account/investor",
        "settrade.openapi.account.marketrep": wrapper_path + "/account/marketrep",
        "settrade.openapi.user": wrapper_path + "/user",
        "settrade.openapi.lib": lib_path,
    },
    package_data={
        "": ["*.so", "*.pyd", "*.dll"]
    },
    python_requires=">=3.5, <3.10",
)
